/*============================================================================
  File:     MergeVSUpsert.sql

  Summary:  The script shows that using the predecessor of MERGE (two 
			statements, INSERT and UPDATE, bounded by a transaction) is 
			non-deterministic when two rows with the same "merge key" appear
			in the source.
			
			MERGE is deterministic and produces an error if two rows 
			with the same "merge key" appear in the source.

  Date:     August 2008

  SQL Server Version: 10.0.1600.22 (RTM)
------------------------------------------------------------------------------
  Written by Bob Beauchemin, SQLskills.com

  For more scripts and sample code, check out http://www.SQLskills.com

  This script is intended only as a supplement to SQL Server Jumpstart program.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

use tempdb
go

--
-- Step 1: Simple example using INSERT AND UPDATE statements
--

IF OBJECTPROPERTY(OBJECT_ID('dbo.target'), 'IsTable') = 1
  drop table dbo.target
GO
IF OBJECTPROPERTY(OBJECT_ID('dbo.staging'), 'IsTable') = 1
  drop table dbo.staging
GO

create table dbo.target (id int, name varchar(20), age int)
go
insert dbo.target values(1, 'Bob', 52)
insert dbo.target values(2, 'Fred', 33)
insert dbo.target values(3, 'Joe', 34)
go

create table dbo.staging (id int, name varchar(20), age int)
insert dbo.staging values(1, 'Bob', 53)
insert dbo.staging values(2, 'Jim', 87)
insert dbo.staging values(5, 'Janet', 45)
go

begin transaction;

-- statement 1
update t
set name = s.name,
    age  = s.age 
from dbo.target t 
join dbo.staging s on (t.id = s.id);

-- statement 2
insert dbo.target 
select * from dbo.staging
where id not in (select id from dbo.target);

commit;

select * from dbo.target
go

--
-- Step 2: Simple example using INSERT AND UPDATE statements
--         This can produce anamolies when multiple dbo.staging rows for same dbo.target row.
--

drop table dbo.target
drop table dbo.staging 

create table dbo.target (id int, name varchar(20), age int)
go
insert dbo.target values(1, 'Bob', 52)
insert dbo.target values(2, 'Fred', 33)
insert dbo.target values(3, 'Joe', 34)
go

create table dbo.staging (id int, name varchar(20), age int)
insert dbo.staging values(1, 'Bob', 53)   -- two rows for ID 1
insert dbo.staging values(1, 'Bill', 55)  -- two rows for ID 1
insert dbo.staging values(2, 'Jim', 87)
insert dbo.staging values(5, 'Janet', 45)
go

-- two rows with an ID of 1 qualify
begin transaction;
-- statemeent 1
update t
set t.name = s.name,
    t.age  = s.age 
from dbo.target t 
join dbo.staging s on (t.id = s.id);

-- statement 2 
insert dbo.target 
select * from dbo.staging
where id not in (select id from dbo.target);

commit;
go

-- updated using one row, one row lost
select * from dbo.target;
go

--
-- Step 3: Simple example using MERGE
--         This cannot produce anamolies when multiple dbo.staging rows for same dbo.target row.
--         Error is produced instead
--

drop table dbo.target
drop table dbo.staging 

create table dbo.target (id int, name varchar(20), age int)
go
insert dbo.target values(1, 'Bob', 52)
insert dbo.target values(2, 'Fred', 33)
insert dbo.target values(3, 'Joe', 34)
go

create table dbo.staging (id int, name varchar(20), age int)
insert dbo.staging values(1, 'Bob', 53)   -- two rows for ID 1
insert dbo.staging values(1, 'Bill', 55)  -- two rows for ID 1
insert dbo.staging values(2, 'Jim', 87)
insert dbo.staging values(5, 'Janet', 45)
go

--
-- Msg 8672, Level 16, State 1, Line 1
-- The MERGE statement attempted to UPDATE or DELETE the same row more than once. This happens when a dbo.target row matches more than one source row. A MERGE statement cannot UPDATE/DELETE the same row of the dbo.target table multiple times. Refine the ON clause to  ensure a dbo.target row matches at most one source row, or use the GROUP BY clause to group the source rows.
merge dbo.target t
  using (select * from dbo.staging) s
  on t.id = s.id
  when matched then
    update set t.name = s.name, t.age = s.age;
go
 